"""
Import compatibility tests for pycfhelpers.

This module tests that all documented import patterns work correctly
and that optional dependencies are handled gracefully.
"""

import pytest
import sys
import importlib


class TestImportCompatibility:
    """Test import compatibility and graceful degradation."""
    
    def test_basic_package_import(self):
        """Test that basic package import works."""
        import pycfhelpers
        
        # Check required attributes exist
        assert hasattr(pycfhelpers, '__version__')
        assert hasattr(pycfhelpers, '__author__')
        assert hasattr(pycfhelpers, '__email__')
        assert hasattr(pycfhelpers, 'CELLFRAME_AVAILABLE')
        assert hasattr(pycfhelpers, 'WEB3_AVAILABLE')
        assert hasattr(pycfhelpers, 'TORNADO_AVAILABLE')
        
        # Check version format
        assert isinstance(pycfhelpers.__version__, str)
        assert len(pycfhelpers.__version__.split('.')) >= 2
    
    def test_common_modules_always_work(self):
        """Test that common modules work without external dependencies."""
        # These should always work regardless of environment
        from common.parsers import parse_cf_v1_address
        from common.types import ChainTypes
        
        # Test basic functionality
        assert hasattr(ChainTypes, 'dag_poa')
        assert callable(parse_cf_v1_address)
    
    def test_logger_fallback(self):
        """Test that logger works with and without CellFrame SDK."""
        from logger import log
        
        # Basic logging methods should exist
        assert hasattr(log, 'info')
        assert hasattr(log, 'error')
        assert hasattr(log, 'warning')
        assert hasattr(log, 'debug')
        assert hasattr(log, 'notice')
        
        # Test that logging doesn't crash
        log.info("Test message")
        log.error("Test error")
    
    def test_helpers_import(self):
        """Test that helpers module imports correctly."""
        import helpers
        
        # Should not raise ImportError
        assert helpers is not None
    
    def test_cellframenet_graceful_failure(self):
        """Test that cellframenet import fails gracefully without CellFrame SDK."""
        try:
            import cellframenet
            # If it imports successfully, that's also OK
            assert cellframenet is not None
        except ImportError as e:
            # Should fail gracefully with a descriptive error
            assert "CellFrame" in str(e) or "DAP" in str(e)
    
    def test_contract_web3_handling(self):
        """Test that contract module handles Web3 dependency correctly."""
        try:
            import contract
            # If Web3 is available, should import successfully
            assert contract is not None
        except ImportError:
            # If Web3 is not available, should fail cleanly
            pass
    
    def test_tornado_http_handling(self):
        """Test that tornado_http handles Tornado dependency correctly."""
        try:
            import tornado_http
            assert tornado_http is not None
        except ImportError:
            # Should fail cleanly if Tornado not available
            pass
    
    def test_optional_dependency_flags(self):
        """Test that optional dependency flags are set correctly."""
        import pycfhelpers
        
        # Flags should be boolean
        assert isinstance(pycfhelpers.CELLFRAME_AVAILABLE, bool)
        assert isinstance(pycfhelpers.WEB3_AVAILABLE, bool)
        assert isinstance(pycfhelpers.TORNADO_AVAILABLE, bool)
        
        # Test consistency with actual imports
        if pycfhelpers.WEB3_AVAILABLE:
            try:
                import web3
                assert web3 is not None
            except ImportError:
                pytest.fail("WEB3_AVAILABLE is True but web3 cannot be imported")
        
        if pycfhelpers.TORNADO_AVAILABLE:
            try:
                import tornado
                assert tornado is not None
            except ImportError:
                pytest.fail("TORNADO_AVAILABLE is True but tornado cannot be imported")
    
    def test_legacy_import_patterns(self):
        """Test all documented legacy import patterns."""
        import_patterns = [
            "import helpers",
            "from logger import log", 
            "from common.parsers import parse_cf_v1_address",
            "from common.types import ChainTypes",
        ]
        
        for pattern in import_patterns:
            try:
                exec(pattern)
                # If it works, great
            except ImportError as e:
                pytest.fail(f"Legacy import pattern '{pattern}' failed: {e}")
    
    def test_node_package_conditional_import(self):
        """Test that node package imports are conditional on CellFrame SDK."""
        import pycfhelpers
        
        if pycfhelpers.CELLFRAME_AVAILABLE:
            # If CellFrame is available, node imports should work
            from node.net import CFNet
            from node.consensus import CFBlock
            assert CFNet is not None
            assert CFBlock is not None
        else:
            # If CellFrame is not available, should fail gracefully
            with pytest.raises(ImportError):
                from node.net import CFNet
    
    def test_package_metadata_consistency(self):
        """Test that package metadata is consistent across modules."""
        import pycfhelpers
        
        # Check that version is consistent
        main_version = pycfhelpers.__version__
        
        # Import duplicate module and check consistency
        import sys
        if 'pycfhelpers' in sys.modules:
            # Reimport should give same version
            import importlib
            importlib.reload(pycfhelpers)
            assert pycfhelpers.__version__ == main_version 