"""
Package integrity tests for pycfhelpers.

This module tests package building, installation, and metadata integrity.
"""

import pytest
import subprocess
import sys
import tempfile
import os
import shutil
from pathlib import Path


class TestPackageIntegrity:
    """Test package integrity and build process."""
    
    def test_package_version_consistency(self):
        """Test that version is consistent across all files."""
        import pycfhelpers
        
        main_version = pycfhelpers.__version__
        
        # Check version in __init__.py if it exists
        try:
            import __init__ as init_module
            if hasattr(init_module, '__version__'):
                assert init_module.__version__ == main_version
        except ImportError:
            pass
        
        # Check that version follows semantic versioning
        version_parts = main_version.split('.')
        assert len(version_parts) >= 2, "Version should have at least major.minor"
        for part in version_parts:
            assert part.isdigit() or 'dev' in part or 'rc' in part or 'a' in part or 'b' in part
    
    def test_required_package_files_exist(self):
        """Test that all required package files exist."""
        required_files = [
            'pyproject.toml',
            'README.md', 
            'LICENSE',
            'MANIFEST.in',
            '__init__.py',
            'pycfhelpers.py',
            'cellframenet.py',
            'helpers.py',
            'logger.py',
        ]
        
        for file_path in required_files:
            assert os.path.exists(file_path), f"Required file {file_path} not found"
    
    def test_manifest_includes_all_necessary_files(self):
        """Test that MANIFEST.in includes all necessary files."""
        with open('MANIFEST.in', 'r') as f:
            manifest_content = f.read()
        
        # Check that key file patterns are included
        assert 'include README.md' in manifest_content
        assert 'include LICENSE' in manifest_content
        assert 'include *.py' in manifest_content
        assert 'recursive-include common *.py' in manifest_content
        assert 'recursive-include node *.py' in manifest_content
        
        # Check that unwanted files are excluded
        assert 'exclude PLAN.md' in manifest_content
        assert 'recursive-exclude * __pycache__' in manifest_content
    
    def test_package_metadata_format(self):
        """Test that package metadata is properly formatted."""
        with open('pyproject.toml', 'r') as f:
            pyproject_content = f.read()
        
        # Check required metadata fields
        assert 'name = "pycfhelpers"' in pyproject_content
        assert 'version =' in pyproject_content
        assert 'description =' in pyproject_content
        assert 'license =' in pyproject_content
        assert 'authors =' in pyproject_content
        
        # Check build system configuration
        assert '[build-system]' in pyproject_content
        assert 'setuptools' in pyproject_content
        assert 'wheel' in pyproject_content
    
    def test_dependencies_specification(self):
        """Test that dependencies are properly specified."""
        with open('pyproject.toml', 'r') as f:
            pyproject_content = f.read()
        
        # Check that base dependencies are minimal
        assert 'base58>=2.0.0' in pyproject_content
        
        # Check optional dependencies structure
        assert '[project.optional-dependencies]' in pyproject_content
        assert 'web3 =' in pyproject_content
        assert 'tornado =' in pyproject_content
        assert 'all =' in pyproject_content
        assert 'dev =' in pyproject_content
    
    def test_python_version_compatibility(self):
        """Test that Python version requirements are reasonable."""
        with open('pyproject.toml', 'r') as f:
            pyproject_content = f.read()
        
        # Should support Python 3.7+
        assert 'requires-python = ">=3.7"' in pyproject_content
        
        # Check classifiers include supported Python versions
        for version in ['3.7', '3.8', '3.9', '3.10', '3.11']:
            assert f'Programming Language :: Python :: {version}' in pyproject_content
    
    def test_package_structure_integrity(self):
        """Test that package structure is correct."""
        # Check that main packages exist
        assert os.path.isdir('common'), "common package directory missing"
        assert os.path.isdir('node'), "node package directory missing"
        
        # Check package __init__.py files
        assert os.path.exists('common/__init__.py'), "common/__init__.py missing"
        assert os.path.exists('node/__init__.py'), "node/__init__.py missing"
        
        # Check key modules exist
        key_modules = [
            'common/types.py',
            'common/parsers.py', 
            'node/net.py',
            'node/consensus.py',
            'node/crypto.py',
            'node/datums.py',
        ]
        
        for module in key_modules:
            assert os.path.exists(module), f"Key module {module} missing"
    
    def test_import_structure_consistency(self):
        """Test that import structure is consistent with package structure."""
        # Test that all py-modules specified in pyproject.toml can be imported
        py_modules = [
            "cellframenet", 
            "contract", 
            "helpers", 
            "logger", 
            "tornado_http",
            "pycfhelpers"
        ]
        
        for module_name in py_modules:
            try:
                __import__(module_name)
                # If import succeeds, great
            except ImportError as e:
                # Some modules may fail due to missing dependencies
                # That's OK as long as they fail gracefully
                assert "CellFrame" in str(e) or "web3" in str(e) or "tornado" in str(e), \
                    f"Module {module_name} failed with unexpected error: {e}"
    
    def test_flake8_configuration(self):
        """Test that flake8 configuration is reasonable."""
        assert os.path.exists('.flake8'), "flake8 configuration missing"
        
        with open('.flake8', 'r') as f:
            flake8_content = f.read()
        
        # Check basic configuration exists
        assert 'max-line-length' in flake8_content
        assert 'ignore' in flake8_content
        assert 'max-complexity' in flake8_content
    
    def test_gitignore_completeness(self):
        """Test that .gitignore covers all necessary patterns."""
        assert os.path.exists('.gitignore'), ".gitignore file missing"
        
        with open('.gitignore', 'r') as f:
            gitignore_content = f.read()
        
        # Check that common Python patterns are ignored
        essential_patterns = [
            '__pycache__',
            '*.pyc',
            '*.pyo',
            'dist/',
            'build/',
            '*.egg-info',
            '.venv',
            'venv/'
        ]
        
        for pattern in essential_patterns:
            assert pattern in gitignore_content, f"Essential gitignore pattern {pattern} missing"
    
    def test_readme_completeness(self):
        """Test that README.md contains essential information."""
        with open('README.md', 'r') as f:
            readme_content = f.read()
        
        # Check essential sections
        essential_sections = [
            '# pycfhelpers',
            '## Installation',
            '## Usage',
            'pip install',
            'import cellframenet',
            'import helpers',
            'from logger import log'
        ]
        
        for section in essential_sections:
            assert section in readme_content, f"Essential README section '{section}' missing"
    
    def test_license_file_exists(self):
        """Test that LICENSE file exists and is not empty."""
        assert os.path.exists('LICENSE'), "LICENSE file missing"
        
        with open('LICENSE', 'r') as f:
            license_content = f.read().strip()
        
        assert len(license_content) > 100, "LICENSE file appears to be empty or too short"
        assert 'MIT' in license_content or 'Apache' in license_content or 'BSD' in license_content, \
            "LICENSE should specify a recognized license type" 