"""Security tests for pycfhelpers."""

import pytest
import os
import re


class TestSecurity:
    """Test security aspects of the package."""
    
    def test_no_hardcoded_secrets(self):
        """Test that there are no hardcoded secrets."""
        secret_patterns = [
            r'password\s*=\s*["\'][^"\']+["\']',
            r'api_key\s*=\s*["\'][^"\']+["\']',
            r'secret\s*=\s*["\'][^"\']+["\']',
        ]
        
        python_files = []
        for root, dirs, files in os.walk('.'):
            dirs[:] = [d for d in dirs if not d.startswith('.') and d not in ['venv', '.venv']]
            for file in files:
                if file.endswith('.py'):
                    python_files.append(os.path.join(root, file))
        
        for file_path in python_files:
            with open(file_path, 'r', encoding='utf-8', errors='ignore') as f:
                content = f.read()
                
            for pattern in secret_patterns:
                if re.search(pattern, content, re.IGNORECASE):
                    pytest.fail(f"Potential hardcoded secret found in {file_path}")
    
    def test_no_dangerous_functions(self):
        """Test that dangerous functions are not used."""
        dangerous_functions = ['eval(', 'exec(']
        
        python_files = []
        for root, dirs, files in os.walk('.'):
            dirs[:] = [d for d in dirs if not d.startswith('.') and d not in ['venv', '.venv']]
            for file in files:
                if file.endswith('.py'):
                    python_files.append(os.path.join(root, file))
        
        for file_path in python_files:
            if 'test_' in file_path:  # Skip test files
                continue
                
            with open(file_path, 'r', encoding='utf-8', errors='ignore') as f:
                content = f.read()
            
            for func in dangerous_functions:
                if func in content and not content.count(f'# {func}') and not content.count(f'"{func}"'):
                    # Allow in test files but warn about production code
                    print(f"WARNING: Dangerous function {func} found in {file_path}")
    
    def test_import_safety(self):
        """Test that imports are safe."""
        python_files = []
        for root, dirs, files in os.walk('.'):
            dirs[:] = [d for d in dirs if not d.startswith('.') and d not in ['venv', '.venv']]
            for file in files:
                if file.endswith('.py'):
                    python_files.append(os.path.join(root, file))
        
        for file_path in python_files:
            with open(file_path, 'r', encoding='utf-8', errors='ignore') as f:
                content = f.read()
            
            # Check for wildcard imports (potential security risk)
            if 'from * import' in content:
                print(f"WARNING: Wildcard import found in {file_path}")
    
    def test_license_compliance(self):
        """Test license compliance."""
        assert os.path.exists('LICENSE'), "LICENSE file must exist"
        
        with open('LICENSE', 'r') as f:
            license_content = f.read()
        
        assert len(license_content) > 50, "LICENSE file seems too short" 