"""
Security and compliance tests for pycfhelpers.

This module tests security aspects and compliance with Python packaging standards.
"""

import pytest
import os
import re
import subprocess
import sys
from pathlib import Path


class TestSecurityCompliance:
    """Test security and compliance aspects."""
    
    def test_no_hardcoded_secrets(self):
        """Test that there are no hardcoded secrets in the code."""
        # Patterns that might indicate hardcoded secrets
        secret_patterns = [
            r'password\s*=\s*["\'][^"\']+["\']',
            r'api_key\s*=\s*["\'][^"\']+["\']',
            r'secret\s*=\s*["\'][^"\']+["\']',
            r'token\s*=\s*["\'][A-Za-z0-9+/]{20,}',
            r'["\'][A-Za-z0-9+/]{32,}["\']',  # Long base64-like strings
        ]
        
        # Files to check
        python_files = []
        for root, dirs, files in os.walk('.'):
            # Skip test files, venv, and git directories
            dirs[:] = [d for d in dirs if not d.startswith('.') and d not in ['venv', '.venv', '__pycache__']]
            for file in files:
                if file.endswith('.py'):
                    python_files.append(os.path.join(root, file))
        
        # Check each file
        for file_path in python_files:
            with open(file_path, 'r', encoding='utf-8', errors='ignore') as f:
                content = f.read()
                
            for pattern in secret_patterns:
                matches = re.finditer(pattern, content, re.IGNORECASE)
                for match in matches:
                    # Allow some common false positives
                    matched_text = match.group().lower()
                    if any(fp in matched_text for fp in ['test', 'example', 'dummy', 'placeholder', '__version__']):
                        continue
                    
                    pytest.fail(f"Potential hardcoded secret found in {file_path}: {match.group()}")
    
    def test_no_eval_or_exec_usage(self):
        """Test that dangerous functions like eval() and exec() are not used unsafely."""
        dangerous_functions = ['eval(', 'exec(', 'compile(']
        
        python_files = []
        for root, dirs, files in os.walk('.'):
            dirs[:] = [d for d in dirs if not d.startswith('.') and d not in ['venv', '.venv', '__pycache__']]
            for file in files:
                if file.endswith('.py'):
                    python_files.append(os.path.join(root, file))
        
        for file_path in python_files:
            with open(file_path, 'r', encoding='utf-8', errors='ignore') as f:
                content = f.read()
            
            for func in dangerous_functions:
                if func in content:
                    # Check if it's in a comment or string literal
                    lines = content.split('\n')
                    for i, line in enumerate(lines, 1):
                        if func in line:
                            stripped = line.strip()
                            # Allow if it's in a comment or test file
                            if (stripped.startswith('#') or 
                                'test_' in file_path or 
                                'for test in import_tests:' in content):
                                continue
                            
                            # This is a more serious finding
                            print(f"WARNING: Potentially dangerous function {func} found in {file_path}:{i}")
                            print(f"Line: {line.strip()}")
    
    def test_import_safety(self):
        """Test that imports are safe and don't execute arbitrary code."""
        # Check for dangerous import patterns
        dangerous_patterns = [
            r'from\s+\*\s+import',  # Wildcard imports can be dangerous
            r'__import__\s*\(',     # Dynamic imports need scrutiny
        ]
        
        python_files = []
        for root, dirs, files in os.walk('.'):
            dirs[:] = [d for d in dirs if not d.startswith('.') and d not in ['venv', '.venv', '__pycache__']]
            for file in files:
                if file.endswith('.py'):
                    python_files.append(os.path.join(root, file))
        
        for file_path in python_files:
            with open(file_path, 'r', encoding='utf-8', errors='ignore') as f:
                content = f.read()
            
            for pattern in dangerous_patterns:
                matches = re.finditer(pattern, content, re.IGNORECASE)
                for match in matches:
                    # Allow some exceptions
                    line = content[content.rfind('\n', 0, match.start()) + 1:content.find('\n', match.end())]
                    if ('test_' in file_path or 
                        line.strip().startswith('#') or
                        '__import__' in pattern and 'test' in line.lower()):
                        continue
                    
                    print(f"WARNING: Potentially unsafe import pattern in {file_path}: {line.strip()}")
    
    def test_file_permissions_security(self):
        """Test that files don't have overly permissive permissions."""
        if os.name == 'nt':  # Skip on Windows
            pytest.skip("File permission test not applicable on Windows")
        
        python_files = []
        for root, dirs, files in os.walk('.'):
            dirs[:] = [d for d in dirs if not d.startswith('.') and d not in ['venv', '.venv', '__pycache__']]
            for file in files:
                if file.endswith('.py'):
                    python_files.append(os.path.join(root, file))
        
        for file_path in python_files:
            stat_info = os.stat(file_path)
            permissions = oct(stat_info.st_mode)[-3:]
            
            # Files should not be world-writable
            if permissions[2] in ['2', '3', '6', '7']:
                pytest.fail(f"File {file_path} is world-writable (permissions: {permissions})")
    
    def test_no_sql_injection_patterns(self):
        """Test for potential SQL injection patterns."""
        sql_patterns = [
            r'["\'].*%s.*["\']',  # String formatting in SQL-like strings
            r'["\'][^"\']*\+.*["\']',  # String concatenation that might be SQL
        ]
        
        python_files = []
        for root, dirs, files in os.walk('.'):
            dirs[:] = [d for d in dirs if not d.startswith('.') and d not in ['venv', '.venv', '__pycache__']]
            for file in files:
                if file.endswith('.py'):
                    python_files.append(os.path.join(root, file))
        
        for file_path in python_files:
            with open(file_path, 'r', encoding='utf-8', errors='ignore') as f:
                content = f.read()
            
            # Look for SQL keywords combined with formatting
            sql_keywords = ['SELECT', 'INSERT', 'UPDATE', 'DELETE', 'DROP', 'CREATE']
            lines = content.split('\n')
            
            for i, line in enumerate(lines, 1):
                upper_line = line.upper()
                if any(keyword in upper_line for keyword in sql_keywords):
                    # Check for string formatting patterns
                    if '%s' in line or '.format(' in line or 'f"' in line or "f'" in line:
                        if not line.strip().startswith('#'):  # Not a comment
                            print(f"WARNING: Potential SQL injection risk in {file_path}:{i}")
                            print(f"Line: {line.strip()}")
    
    def test_dependency_versions_pinned(self):
        """Test that dependencies have version constraints."""
        with open('pyproject.toml', 'r') as f:
            content = f.read()
        
        # Extract dependency specifications
        import re
        dep_pattern = r'"([^"]+)>=([^"]+)"'
        dependencies = re.findall(dep_pattern, content)
        
        for dep_name, version in dependencies:
            # Check that version constraints are reasonable
            version_parts = version.split('.')
            assert len(version_parts) >= 2, f"Dependency {dep_name} should have at least major.minor version"
            
            # Major version should be specified
            assert version_parts[0].isdigit(), f"Dependency {dep_name} should have numeric major version"
    
    def test_no_debug_code_in_production(self):
        """Test that debug code is not left in production."""
        debug_patterns = [
            r'print\s*\(',
            r'pdb\.set_trace\(\)',
            r'breakpoint\(\)',
            r'debugger',
            r'console\.log',
        ]
        
        python_files = []
        for root, dirs, files in os.walk('.'):
            dirs[:] = [d for d in dirs if not d.startswith('.') and d not in ['venv', '.venv', '__pycache__']]
            for file in files:
                if file.endswith('.py'):
                    python_files.append(os.path.join(root, file))
        
        for file_path in python_files:
            # Skip test files and certain modules where print is expected
            if ('test_' in file_path or 
                'logger.py' in file_path or 
                '__init__.py' in file_path):
                continue
                
            with open(file_path, 'r', encoding='utf-8', errors='ignore') as f:
                content = f.read()
            
            lines = content.split('\n')
            for i, line in enumerate(lines, 1):
                for pattern in debug_patterns:
                    if re.search(pattern, line, re.IGNORECASE):
                        stripped = line.strip()
                        # Skip comments and documented examples
                        if (stripped.startswith('#') or 
                            'example' in stripped.lower() or
                            'test' in stripped.lower()):
                            continue
                        
                        print(f"WARNING: Potential debug code in {file_path}:{i}")
                        print(f"Line: {stripped}")
    
    def test_license_compliance(self):
        """Test that license is properly specified and consistent."""
        # Check LICENSE file exists
        assert os.path.exists('LICENSE'), "LICENSE file must exist"
        
        # Check pyproject.toml has license specified
        with open('pyproject.toml', 'r') as f:
            pyproject_content = f.read()
        
        assert 'license =' in pyproject_content, "License must be specified in pyproject.toml"
        
        # Check that license is recognized
        with open('LICENSE', 'r') as f:
            license_content = f.read()
        
        recognized_licenses = ['MIT', 'Apache', 'BSD', 'GPL', 'LGPL']
        assert any(license_type in license_content for license_type in recognized_licenses), \
            "LICENSE file should contain a recognized license type"
    
    def test_metadata_security(self):
        """Test that package metadata doesn't contain sensitive information."""
        with open('pyproject.toml', 'r') as f:
            content = f.read()
        
        # Check for potentially sensitive patterns in metadata
        sensitive_patterns = [
            r'password',
            r'secret',
            r'token',
            r'localhost:\d+',
            r'127\.0\.0\.1',
            r'192\.168\.',
            r'10\.\d+\.\d+\.\d+',
        ]
        
        for pattern in sensitive_patterns:
            matches = re.finditer(pattern, content, re.IGNORECASE)
            for match in matches:
                # Check context - allow some patterns in URLs and descriptions
                context = content[max(0, match.start()-50):match.end()+50]
                if ('gitlab.demlabs.net' in context or 
                    'description' in context or
                    'homepage' in context):
                    continue
                
                print(f"WARNING: Potentially sensitive information in metadata: {match.group()}")
    
    def test_code_injection_prevention(self):
        """Test that code doesn't use patterns prone to injection attacks."""
        injection_patterns = [
            r'os\.system\s*\(',
            r'subprocess\.call\s*\(',
            r'subprocess\.run\s*\([^)]*shell\s*=\s*True',
            r'os\.popen\s*\(',
        ]
        
        python_files = []
        for root, dirs, files in os.walk('.'):
            dirs[:] = [d for d in dirs if not d.startswith('.') and d not in ['venv', '.venv', '__pycache__']]
            for file in files:
                if file.endswith('.py'):
                    python_files.append(os.path.join(root, file))
        
        for file_path in python_files:
            with open(file_path, 'r', encoding='utf-8', errors='ignore') as f:
                content = f.read()
            
            for pattern in injection_patterns:
                matches = re.finditer(pattern, content)
                for match in matches:
                    line_start = content.rfind('\n', 0, match.start()) + 1
                    line_end = content.find('\n', match.end())
                    if line_end == -1:
                        line_end = len(content)
                    line = content[line_start:line_end]
                    
                    # Skip if it's a comment or in test files
                    if (line.strip().startswith('#') or 
                        'test_' in file_path):
                        continue
                    
                    print(f"WARNING: Potentially unsafe system call in {file_path}")
                    print(f"Pattern: {match.group()}")
                    print(f"Line: {line.strip()}") 